@@ -15,6 +15,7 @@ class MetamodelBuilderTest < Test::Unit::TestCase
       has_attr 'name' # default is String
       has_attr 'stringWithDefault', String, :defaultValueLiteral => "xtest"
       has_attr 'integerWithDefault', Integer, :defaultValueLiteral => "123"
+      has_attr 'longWithDefault', Long, :defaultValueLiteral => "1234567890"
       has_attr 'floatWithDefault', Float, :defaultValueLiteral => "0.123"
       has_attr 'boolWithDefault', Boolean, :defaultValueLiteral => "true"
       has_attr 'anything', Object
@@ -148,9 +149,17 @@ class MetamodelBuilderTest < Test::Unit::TestCase
 
     class ContainerClass < RGen::MetamodelBuilder::MMBase
       contains_one_uni 'oneChildUni', ContainedClass
+      contains_one_uni 'oneChildUni2', ContainedClass
       contains_one 'oneChild', ContainedClass, 'parentOne'
+      contains_one 'oneChild2', ContainedClass, 'parentOne2'
       contains_many_uni 'manyChildUni', ContainedClass
+      contains_many_uni 'manyChildUni2', ContainedClass
       contains_many 'manyChild', ContainedClass, 'parentMany'
+      contains_many 'manyChild2', ContainedClass, 'parentMany2'
+    end
+
+    class NestedContainerClass < ContainedClass
+      contains_one_uni 'oneChildUni', ContainedClass
     end
 
     class OppositeRefAssocA < RGen::MetamodelBuilder::MMBase
@@ -183,6 +192,7 @@ class MetamodelBuilderTest < Test::Unit::TestCase
     assert_equal "xtest", sc.stringWithDefault
     assert_equal :extended, sc.kindWithDefault
     assert_equal 123, sc.integerWithDefault
+    assert_equal 1234567890, sc.longWithDefault
     assert_equal 0.123, sc.floatWithDefault
     assert_equal true, sc.boolWithDefault
 
@@ -253,6 +263,20 @@ class MetamodelBuilderTest < Test::Unit::TestCase
     assert sc2.floatWithDefault.is_a?(BigDecimal)
     assert_equal "123456789012345678.0", sc2.floatWithDefault.to_s("F")
   end
+
+  def test_long
+    sc = mm::SimpleClass.new
+    sc.longWithDefault = 5
+    assert_equal 5, sc.longWithDefault
+    sc.longWithDefault = 1234567890
+    assert_equal 1234567890, sc.longWithDefault
+    assert sc.longWithDefault.is_a?(Bignum)
+    assert sc.longWithDefault.is_a?(Integer)
+    err = assert_raise StandardError do
+      sc.longWithDefault = "a string"
+    end
+    assert_match /In (\w+::)+SimpleClass : Can not use a String where a Integer is expected/, err.message
+  end
   
   def test_many_attr
     o = mm::ManyAttrClass.new
@@ -807,6 +831,8 @@ class MetamodelBuilderTest < Test::Unit::TestCase
     assert_equal [e1], e2.aClasses
   end
   
+  # Multiplicity agnostic convenience methods
+
   def test_genericAccess
     e1 = mm::OneClass.new
     e2 = mm::ManyClass.new
@@ -826,6 +852,40 @@ class MetamodelBuilderTest < Test::Unit::TestCase
     assert_equal [], e4.getGenericAsArray("oneClass")
   end
 
+  def test_setNilOrRemoveGeneric
+    e1 = mm::OneClass.new
+    e2 = mm::ManyClass.new
+    e3 = mm::OneClass.new
+    # use on "many" feature
+    e1.addManyClasses(e2)
+    assert_equal [e2], e1.manyClasses
+    e1.setNilOrRemoveGeneric("manyClasses", e2)
+    assert_equal [], e1.manyClasses
+    # use on "one" feature
+    e2.oneClass = e3
+    assert_equal e3, e2.oneClass
+    e2.setNilOrRemoveGeneric("oneClass", e3)
+    assert_nil e2.oneClass
+  end
+
+  def test_setNilOrRemoveAllGeneric
+    e1 = mm::OneClass.new
+    e2 = mm::ManyClass.new
+    e3 = mm::OneClass.new
+    e4 = mm::ManyClass.new
+    # use on "many" feature
+    e1.addManyClasses(e2)
+    e1.addManyClasses(e4)
+    assert_equal [e2, e4], e1.manyClasses
+    e1.setNilOrRemoveAllGeneric("manyClasses")
+    assert_equal [], e1.manyClasses
+    # use on "one" feature
+    e2.oneClass = e3
+    assert_equal e3, e2.oneClass
+    e2.setNilOrRemoveAllGeneric("oneClass")
+    assert_nil e2.oneClass
+  end
+
   def test_abstract
     err = assert_raise StandardError do
       mm::AbstractClass.new
@@ -871,6 +931,11 @@ class MetamodelBuilderTest < Test::Unit::TestCase
         has_attr 'enumWithDefault', kindType, :defaultValueLiteral => "7"
       end
     end
+    Test8 = proc do 
+      class BadClass < RGen::MetamodelBuilder::MMBase
+        has_attr 'longWithDefault', Integer, :defaultValueLiteral => "1.1"
+      end
+    end
   end
 
   def test_bad_default_value_literal
@@ -902,6 +967,10 @@ class MetamodelBuilderTest < Test::Unit::TestCase
       BadDefaultValueLiteralContainer::Test7.call
     end
     assert_equal "Property enumWithDefault can not take value 7, expected one of :simple, :extended", err.message
+    err = assert_raise StandardError do
+      BadDefaultValueLiteralContainer::Test8.call
+    end
+    assert_equal "Property longWithDefault can not take value 1.1, expected an Integer", err.message
   end
 
   def test_isset_set_to_nil
@@ -1017,39 +1086,57 @@ class MetamodelBuilderTest < Test::Unit::TestCase
     a = mm::ContainerClass.new
     b = mm::ContainedClass.new
     c = mm::ContainedClass.new
+    assert_equal [], a.eContents
+    assert_equal [], a.eAllContents
     assert_nil b.eContainer
     assert_nil b.eContainingFeature
     a.oneChildUni = b
     assert_equal a, b.eContainer
     assert_equal :oneChildUni, b.eContainingFeature
+    assert_equal [b], a.eContents
+    assert_equal [b], a.eAllContents
     a.oneChildUni = c
     assert_nil b.eContainer
     assert_nil b.eContainingFeature
     assert_equal a, c.eContainer
     assert_equal :oneChildUni, c.eContainingFeature
+    assert_equal [c], a.eContents
+    assert_equal [c], a.eAllContents
     a.oneChildUni = nil
     assert_nil c.eContainer
     assert_nil c.eContainingFeature
+    assert_equal [], a.eContents
+    assert_equal [], a.eAllContents
   end
 
   def test_container_many_uni
     a = mm::ContainerClass.new
     b = mm::ContainedClass.new
     c = mm::ContainedClass.new
+    assert_equal [], a.eContents
+    assert_equal [], a.eAllContents
     a.addManyChildUni(b)
     assert_equal a, b.eContainer
     assert_equal :manyChildUni, b.eContainingFeature
+    assert_equal [b], a.eContents
+    assert_equal [b], a.eAllContents
     a.addManyChildUni(c)
     assert_equal a, c.eContainer
     assert_equal :manyChildUni, c.eContainingFeature
+    assert_equal [b, c], a.eContents
+    assert_equal [b, c], a.eAllContents
     a.removeManyChildUni(b)
     assert_nil b.eContainer
     assert_nil b.eContainingFeature
     assert_equal a, c.eContainer
     assert_equal :manyChildUni, c.eContainingFeature
+    assert_equal [c], a.eContents
+    assert_equal [c], a.eAllContents
     a.removeManyChildUni(c)
     assert_nil c.eContainer
     assert_nil c.eContainingFeature
+    assert_equal [], a.eContents
+    assert_equal [], a.eAllContents
   end
 
   def test_conainer_one_bi
@@ -1060,14 +1147,22 @@ class MetamodelBuilderTest < Test::Unit::TestCase
     a.oneChild = b
     assert_equal a, b.eContainer
     assert_equal :oneChild, b.eContainingFeature
+    assert_equal [b], a.eContents
+    assert_equal [b], a.eAllContents
     c.oneChild = d 
     assert_equal c, d.eContainer
     assert_equal :oneChild, d.eContainingFeature
+    assert_equal [d], c.eContents
+    assert_equal [d], c.eAllContents
     a.oneChild = d
     assert_nil b.eContainer
     assert_nil b.eContainingFeature
     assert_equal a, d.eContainer
     assert_equal :oneChild, d.eContainingFeature
+    assert_equal [d], a.eContents
+    assert_equal [d], a.eAllContents
+    assert_equal [], c.eContents
+    assert_equal [], c.eAllContents
   end
 
   def test_conainer_one_bi_rev
@@ -1078,14 +1173,22 @@ class MetamodelBuilderTest < Test::Unit::TestCase
     a.oneChild = b
     assert_equal a, b.eContainer
     assert_equal :oneChild, b.eContainingFeature
+    assert_equal [b], a.eContents
+    assert_equal [b], a.eAllContents
     c.oneChild = d 
     assert_equal c, d.eContainer
     assert_equal :oneChild, d.eContainingFeature
+    assert_equal [d], c.eContents
+    assert_equal [d], c.eAllContents
     d.parentOne = a
     assert_nil b.eContainer
     assert_nil b.eContainingFeature
     assert_equal a, d.eContainer
     assert_equal :oneChild, d.eContainingFeature
+    assert_equal [d], a.eContents
+    assert_equal [d], a.eAllContents
+    assert_equal [], c.eContents
+    assert_equal [], c.eAllContents
   end
 
   def test_conainer_one_bi_nil
@@ -1094,9 +1197,13 @@ class MetamodelBuilderTest < Test::Unit::TestCase
     a.oneChild = b
     assert_equal a, b.eContainer
     assert_equal :oneChild, b.eContainingFeature
+    assert_equal [b], a.eContents
+    assert_equal [b], a.eAllContents
     a.oneChild = nil 
     assert_nil b.eContainer
     assert_nil b.eContainingFeature
+    assert_equal [], a.eContents
+    assert_equal [], a.eAllContents
   end
 
   def test_conainer_one_bi_nil_rev
@@ -1105,9 +1212,13 @@ class MetamodelBuilderTest < Test::Unit::TestCase
     a.oneChild = b
     assert_equal a, b.eContainer
     assert_equal :oneChild, b.eContainingFeature
+    assert_equal [b], a.eContents
+    assert_equal [b], a.eAllContents
     b.parentOne = nil 
     assert_nil b.eContainer
     assert_nil b.eContainingFeature
+    assert_equal [], a.eContents
+    assert_equal [], a.eAllContents
   end
 
   def test_container_many_bi
@@ -1120,9 +1231,13 @@ class MetamodelBuilderTest < Test::Unit::TestCase
     assert_equal :manyChild, b.eContainingFeature
     assert_equal a, c.eContainer
     assert_equal :manyChild, c.eContainingFeature
+    assert_equal [b, c], a.eContents
+    assert_equal [b, c], a.eAllContents
     a.removeManyChild(b)
     assert_nil b.eContainer
     assert_nil b.eContainingFeature
+    assert_equal [c], a.eContents
+    assert_equal [c], a.eAllContents
   end
 
   def test_conainer_many_bi_steal
@@ -1136,12 +1251,18 @@ class MetamodelBuilderTest < Test::Unit::TestCase
     assert_equal :manyChild, b.eContainingFeature
     assert_equal a, c.eContainer
     assert_equal :manyChild, c.eContainingFeature
+    assert_equal [b, c], a.eContents
+    assert_equal [b, c], a.eAllContents
     d.addManyChild(b)
     assert_equal d, b.eContainer
     assert_equal :manyChild, b.eContainingFeature
+    assert_equal [c], a.eContents
+    assert_equal [c], a.eAllContents
+    assert_equal [b], d.eContents
+    assert_equal [b], d.eAllContents
   end
 
-  def test_conainer_many_bi_steal_ref
+  def test_conainer_many_bi_steal_rev
     a = mm::ContainerClass.new
     b = mm::ContainedClass.new
     c = mm::ContainedClass.new
@@ -1152,9 +1273,51 @@ class MetamodelBuilderTest < Test::Unit::TestCase
     assert_equal :manyChild, b.eContainingFeature
     assert_equal a, c.eContainer
     assert_equal :manyChild, c.eContainingFeature
+    assert_equal [b, c], a.eContents
+    assert_equal [b, c], a.eAllContents
     b.parentMany = d
     assert_equal d, b.eContainer
     assert_equal :manyChild, b.eContainingFeature
+    assert_equal [c], a.eContents
+    assert_equal [c], a.eAllContents
+    assert_equal [b], d.eContents
+    assert_equal [b], d.eAllContents
+  end
+
+  def test_all_contents
+    a = mm::ContainerClass.new
+    b = mm::NestedContainerClass.new
+    c = mm::ContainedClass.new
+    a.oneChildUni = b
+    b.oneChildUni = c
+    assert_equal [b, c], a.eAllContents
+  end
+
+  def test_all_contents_with_block
+    a = mm::ContainerClass.new
+    b = mm::NestedContainerClass.new
+    c = mm::ContainedClass.new
+    a.oneChildUni = b
+    b.oneChildUni = c
+    yielded = []
+    a.eAllContents do |e|
+      yielded << e
+    end
+    assert_equal [b, c], yielded
+  end
+
+  def test_all_contents_prune
+    a = mm::ContainerClass.new
+    b = mm::NestedContainerClass.new
+    c = mm::ContainedClass.new
+    a.oneChildUni = b
+    b.oneChildUni = c
+    yielded = []
+    a.eAllContents do |e|
+      yielded << e
+      :prune
+    end
+    assert_equal [b], yielded
   end
 
   def test_container_generic
@@ -1198,4 +1361,122 @@ class MetamodelBuilderTest < Test::Unit::TestCase
     assert_equal [], a.manyChildUni
   end
 
+  def test_disconnectContainer_one_uni
+    a = mm::ContainerClass.new
+    b = mm::ContainedClass.new
+    a.oneChildUni = b
+    b.disconnectContainer
+    assert_nil a.oneChildUni
+  end
+
+  def test_disconnectContainer_one
+    a = mm::ContainerClass.new
+    b = mm::ContainedClass.new
+    a.oneChild = b
+    b.disconnectContainer
+    assert_nil a.oneChild
+    assert_nil b.parentOne
+  end
+
+  def test_disconnectContainer_many_uni
+    a = mm::ContainerClass.new
+    b = mm::ContainedClass.new
+    c = mm::ContainedClass.new
+    a.addManyChildUni(b)
+    a.addManyChildUni(c)
+    b.disconnectContainer
+    assert_equal [c], a.manyChildUni
+  end
+
+  def test_disconnectContainer_many
+    a = mm::ContainerClass.new
+    b = mm::ContainedClass.new
+    c = mm::ContainedClass.new
+    a.addManyChild(b)
+    a.addManyChild(c)
+    b.disconnectContainer
+    assert_nil b.parentMany
+    assert_equal [c], a.manyChild
+  end
+
+  # Duplicate Containment Tests
+  #
+  # Testing that no element is contained in two different containers at a time.
+  # This must also work for uni-directional containments as well as
+  # for containments via different roles.
+
+  # here the bi-dir reference disconnects from the previous container
+  def test_duplicate_containment_bidir_samerole_one
+    a1 = mm::ContainerClass.new
+    a2 = mm::ContainerClass.new
+    b = mm::ContainedClass.new
+    a1.oneChild = b
+    a2.oneChild = b
+    assert_nil a1.oneChild
+  end
+
+  # here the bi-dir reference disconnects from the previous container
+  def test_duplicate_containment_bidir_samerole_many
+    a1 = mm::ContainerClass.new
+    a2 = mm::ContainerClass.new
+    b = mm::ContainedClass.new
+    a1.addManyChild(b)
+    a2.addManyChild(b)
+    assert_equal [], a1.manyChild
+  end
+
+  def test_duplicate_containment_unidir_samerole_one
+    a1 = mm::ContainerClass.new
+    a2 = mm::ContainerClass.new
+    b = mm::ContainedClass.new
+    a1.oneChildUni = b
+    a2.oneChildUni = b
+    assert_nil a1.oneChildUni
+  end
+
+  def test_duplicate_containment_unidir_samerole_many
+    a1 = mm::ContainerClass.new
+    a2 = mm::ContainerClass.new
+    b = mm::ContainedClass.new
+    a1.addManyChildUni(b)
+    a2.addManyChildUni(b)
+    assert_equal [], a1.manyChildUni
+  end
+
+  def test_duplicate_containment_bidir_otherrole_one
+    a1 = mm::ContainerClass.new
+    a2 = mm::ContainerClass.new
+    b = mm::ContainedClass.new
+    a1.oneChild = b
+    a2.oneChild2 = b
+    assert_nil a1.oneChild
+  end
+
+  def test_duplicate_containment_bidir_otherrole_many
+    a1 = mm::ContainerClass.new
+    a2 = mm::ContainerClass.new
+    b = mm::ContainedClass.new
+    a1.addManyChild(b)
+    a2.addManyChild2(b)
+    assert_equal [], a1.manyChild
+  end
+
+  def test_duplicate_containment_unidir_otherrole_one
+    a1 = mm::ContainerClass.new
+    a2 = mm::ContainerClass.new
+    b = mm::ContainedClass.new
+    a1.oneChildUni = b
+    a2.oneChildUni2 = b
+    assert_nil a1.oneChildUni
+  end
+
+  def test_duplicate_containment_unidir_otherrole_many
+    a1 = mm::ContainerClass.new
+    a2 = mm::ContainerClass.new
+    b = mm::ContainedClass.new
+    a1.addManyChildUni(b)
+    a2.addManyChildUni2(b)
+    assert_equal [], a1.manyChildUni
+  end
+
 end
